@@ -37,14 +37,13 @@ namespace Plasma
 {
 
 ContainmentActions::ContainmentActions(QObject *parentObject)
-    : d(new ContainmentActionsPrivate(KService::serviceByStorageId(QString()), this))
+    : d(new ContainmentActionsPrivate({}, this))
 {
     setParent(parentObject);
 }
 
 ContainmentActions::ContainmentActions(QObject *parentObject, const QVariantList &args)
-    : d(new ContainmentActionsPrivate(KService::serviceByStorageId(args.count() > 0 ?
-                                      args[0].toString() : QString()), this))
+    : d(new ContainmentActionsPrivate(args.value(0), this))
 {
     setParent(parentObject);
 
@@ -63,6 +62,11 @@ ContainmentActions::~ContainmentActions()
 }
 
 KPluginInfo ContainmentActions::pluginInfo() const
+{
+    return KPluginInfo(d->containmentActionsDescription);
+}
+
+KPluginMetaData ContainmentActions::metadata() const
 {
     return d->containmentActionsDescription;
 }
@@ -132,7 +136,13 @@ QString ContainmentActions::eventToString(QEvent *event)
         int o = QObject::staticQtMetaObject.indexOfEnumerator("Orientations");
         QMetaEnum orient = QObject::staticQtMetaObject.enumerator(o);
         trigger = QStringLiteral("wheel:");
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
         trigger += QString::fromLatin1(orient.valueToKey(e->orientation()));
+#else
+        // ContainmentInterface::wheelEvent uses angleDelta.y()
+        // To support both, should we just remove this orientation string?
+        trigger += QStringLiteral("Vertical");
+#endif
         modifiers = e->modifiers();
         break;
     }
