@@ -54,13 +54,13 @@ Theme::Theme(QObject *parent)
     if (!ThemePrivate::globalTheme) {
         ThemePrivate::globalTheme = new ThemePrivate;
     }
-    ThemePrivate::globalThemeRefCount.ref();
+    ThemePrivate::globalTheme->ref.ref();
     d = ThemePrivate::globalTheme;
 
     d->settingsChanged(false);
     if (QCoreApplication::instance()) {
-        connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
-                d, SLOT(onAppExitCleanup()));
+        connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
+                d, &ThemePrivate::onAppExitCleanup);
     }
     connect(d, &ThemePrivate::themeChanged, this, &Theme::themeChanged);
     connect(d, &ThemePrivate::defaultFontChanged, this, &Theme::defaultFontChanged);
@@ -70,13 +70,13 @@ Theme::Theme(QObject *parent)
 Theme::Theme(const QString &themeName, QObject *parent)
     : QObject(parent)
 {
-    if (!ThemePrivate::themes.contains(themeName)) {
-        ThemePrivate::themes[themeName] = new ThemePrivate;
-        ThemePrivate::themesRefCount[themeName] = QAtomicInt();
+    auto& priv = ThemePrivate::themes[themeName];
+    if (!priv) {
+        priv = new ThemePrivate;
     }
 
-    ThemePrivate::themesRefCount[themeName].ref();
-    d = ThemePrivate::themes[themeName];
+    priv->ref.ref();
+    d = priv;
 
     // turn off caching so we don't accidentally trigger unnecessary disk activity at this point
     bool useCache = d->cacheTheme;
@@ -85,8 +85,8 @@ Theme::Theme(const QString &themeName, QObject *parent)
     d->cacheTheme = useCache;
     d->fixedName = true;
     if (QCoreApplication::instance()) {
-        connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
-                d, SLOT(onAppExitCleanup()));
+        connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
+                d, &ThemePrivate::onAppExitCleanup);
     }
     connect(d, &ThemePrivate::themeChanged, this, &Theme::themeChanged);
 }
@@ -94,18 +94,15 @@ Theme::Theme(const QString &themeName, QObject *parent)
 Theme::~Theme()
 {
     if (d == ThemePrivate::globalTheme) {
-        if (!ThemePrivate::globalThemeRefCount.deref()) {
+        if (!d->ref.deref()) {
             disconnect(ThemePrivate::globalTheme, nullptr, this, nullptr);
             delete ThemePrivate::globalTheme;
             ThemePrivate::globalTheme = nullptr;
             d = nullptr;
         }
     } else {
-        if (!ThemePrivate::themesRefCount[d->themeName].deref()) {
-            ThemePrivate *themePrivate = ThemePrivate::themes[d->themeName];
-            ThemePrivate::themes.remove(d->themeName);
-            ThemePrivate::themesRefCount.remove(d->themeName);
-            delete themePrivate;
+        if (!d->ref.deref()) {
+            delete ThemePrivate::themes.take(d->themeName);
         }
     }
 }
@@ -118,21 +115,19 @@ void Theme::setThemeName(const QString &themeName)
 
     if (d != ThemePrivate::globalTheme) {
         disconnect(QCoreApplication::instance(), nullptr, d, nullptr);
-        if (!ThemePrivate::themesRefCount[d->themeName].deref()) {
-            ThemePrivate *themePrivate = ThemePrivate::themes[d->themeName];
-            ThemePrivate::themes.remove(d->themeName);
-            ThemePrivate::themesRefCount.remove(d->themeName);
-            delete themePrivate;
+        if (!d->ref.deref()) {
+            delete ThemePrivate::themes.take(d->themeName);
         }
-        if (!ThemePrivate::themes.contains(themeName)) {
-            ThemePrivate::themes[themeName] = new ThemePrivate;
-            ThemePrivate::themesRefCount[themeName] = QAtomicInt();
+
+        auto& priv = ThemePrivate::themes[themeName];
+        if (!priv) {
+            priv = new ThemePrivate;
         }
-        ThemePrivate::themesRefCount[themeName].ref();
-        d = ThemePrivate::themes[themeName];
+        priv->ref.ref();
+        d = priv;
         if (QCoreApplication::instance()) {
-            connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
-                    d, SLOT(onAppExitCleanup()));
+            connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
+                    d, &ThemePrivate::onAppExitCleanup);
         }
         connect(d, &ThemePrivate::themeChanged, this, &Theme::themeChanged);
     }
@@ -154,12 +149,12 @@ QString Theme::imagePath(const QString &name) const
         return QString();
     }
 
-    const QString svgzName = name % QLatin1Literal(".svgz");
+    const QString svgzName = name % QLatin1String(".svgz");
     QString path = d->findInTheme(svgzName, d->themeName);
 
     if (path.isEmpty()) {
         // try for an uncompressed svg file
-        const QString svgName = name % QLatin1Literal(".svg");
+        const QString svgName = name % QLatin1String(".svg");
         path = d->findInTheme(svgName, d->themeName);
 
         // search in fallback themes if necessary
@@ -220,10 +215,10 @@ QString Theme::wallpaperPath(const QSize &size) const
     //      to override the theme?
     if (d->hasWallpapers) {
         // check in the theme first
-        fullPath = d->findInTheme(QLatin1Literal("wallpapers/") % image, d->themeName);
+        fullPath = d->findInTheme(QLatin1String("wallpapers/") % image, d->themeName);
 
         if (fullPath.isEmpty()) {
-            fullPath = d->findInTheme(QLatin1Literal("wallpapers/") % defaultImage, d->themeName);
+            fullPath = d->findInTheme(QLatin1String("wallpapers/") % defaultImage, d->themeName);
         }
     }
 
@@ -261,8 +256,8 @@ bool Theme::currentThemeHasImage(const QString &name) const
         return false;
     }
 
-    return !(d->findInTheme(name % QLatin1Literal(".svgz"), d->themeName, false).isEmpty()) ||
-           !(d->findInTheme(name % QLatin1Literal(".svg"), d->themeName, false).isEmpty());
+    return !(d->findInTheme(name % QLatin1String(".svgz"), d->themeName, false).isEmpty()) ||
+           !(d->findInTheme(name % QLatin1String(".svg"), d->themeName, false).isEmpty());
 }
 
 KSharedConfigPtr Theme::colorScheme() const
@@ -294,14 +289,15 @@ bool Theme::useGlobalSettings() const
 
 bool Theme::findInCache(const QString &key, QPixmap &pix, unsigned int lastModified)
 {
-    if (lastModified != 0 && d->useCache() && lastModified > uint(d->pixmapCache->lastModifiedTime().toTime_t())) {
+    if (lastModified != 0 && d->useCache() && lastModified > uint(d->pixmapCache->lastModifiedTime().toSecsSinceEpoch())) {
         return false;
     }
 
     if (d->useCache()) {
         const QString id = d->keysToCache.value(key);
-        if (d->pixmapsToCache.contains(id)) {
-            pix = d->pixmapsToCache.value(id);
+        const auto it = d->pixmapsToCache.constFind(id);
+        if (it != d->pixmapsToCache.constEnd()) {
+            pix = *it;
             return !pix.isNull();
         }
 
@@ -325,14 +321,9 @@ void Theme::insertIntoCache(const QString &key, const QPixmap &pix)
 void Theme::insertIntoCache(const QString &key, const QPixmap &pix, const QString &id)
 {
     if (d->useCache()) {
-        d->pixmapsToCache.insert(id, pix);
-
-        if (d->idsToCache.contains(id)) {
-            d->keysToCache.remove(d->idsToCache[id]);
-        }
-
-        d->keysToCache.insert(key, id);
-        d->idsToCache.insert(id, key);
+        d->pixmapsToCache[id] = pix;
+        d->keysToCache[key] = id;
+        d->idsToCache[id] = key;
 
         //always start timer in d->pixmapSaveTimer's thread
         QMetaObject::invokeMethod(d->pixmapSaveTimer, "start", Qt::QueuedConnection);
@@ -346,7 +337,7 @@ bool Theme::findInRectsCache(const QString &image, const QString &element, QRect
     }
 
     KConfigGroup imageGroup(d->svgElementsCache, image);
-    rect = imageGroup.readEntry(element % QLatin1Literal("Size"), QRectF());
+    rect = imageGroup.readEntry(element % QLatin1String("Size"), QRectF());
 
     if (rect.isValid()) {
         return true;
@@ -403,7 +394,7 @@ void Theme::insertIntoRectsCache(const QString &image, const QString &element, c
 
     if (rect.isValid()) {
         KConfigGroup imageGroup(d->svgElementsCache, image);
-        imageGroup.writeEntry(element % QLatin1Literal("Size"), rect);
+        imageGroup.writeEntry(element % QLatin1String("Size"), rect);
     } else {
         QHash<QString, QSet<QString> >::iterator it = d->invalidElements.find(image);
         if (it == d->invalidElements.end()) {
@@ -436,7 +427,7 @@ void Theme::releaseRectsCache(const QString &image)
     if (it != d->invalidElements.end()) {
         if (d->useCache()) {
             KConfigGroup imageGroup(d->svgElementsCache, it.key());
-            imageGroup.writeEntry("invalidElements", it.value().toList());
+            imageGroup.writeEntry("invalidElements", it.value().values());
         }
 
         d->invalidElements.erase(it);
@@ -452,7 +443,7 @@ void Theme::setCacheLimit(int kbytes)
 
 KPluginInfo Theme::pluginInfo() const
 {
-    return d->pluginInfo;
+    return KPluginInfo(d->pluginMetaData);
 }
 
 QFont Theme::defaultFont() const
@@ -477,19 +468,44 @@ bool Theme::backgroundContrastEnabled() const
 
 qreal Theme::backgroundContrast() const
 {
+    if (qIsNaN(d->backgroundContrast)) {
+        // Make up sensible default values, based on the background color
+        // If we're using a dark background color, darken the background
+        if (qGray(color(Plasma::Theme::BackgroundColor).rgb()) < 127) {
+            return 0.45;
+        // for a light theme lighten up the background
+        } else {
+            return 0.3;
+        }
+    }
     return d->backgroundContrast;
 }
 
 qreal Theme::backgroundIntensity() const
 {
+    if (qIsNaN(d->backgroundIntensity)) {
+        if (qGray(color(Plasma::Theme::BackgroundColor).rgb()) < 127) {
+            return 0.45;
+        } else {
+            return 1.9;
+        }
+    }
     return d->backgroundIntensity;
 }
 
 qreal Theme::backgroundSaturation() const
 {
+    if (qIsNaN(d->backgroundSaturation)) {
+        return 1.7;
+    }
     return d->backgroundSaturation;
 }
 
+bool Theme::blurBehindEnabled() const
+{
+    return d->blurBehindEnabled;
+}
+
 }
 
 #include "moc_theme.cpp"
