@@ -48,6 +48,7 @@
 #include <QDebug>
 
 #if HAVE_KWAYLAND
+#include "waylandintegration_p.h"
 #include <KWayland/Client/plasmashell.h>
 #include <KWayland/Client/surface.h>
 #endif
@@ -57,6 +58,10 @@
 #include <xcb/shape.h>
 #endif
 
+#if HAVE_X11
+#include <QtPlatformHeaders/QXcbWindowFunctions>
+#endif
+
 //Unfortunately QWINDOWSIZE_MAX is not exported
 #define DIALOGSIZE_MAX ((1<<24)-1)
 
@@ -139,6 +144,7 @@ public:
 
     void setupWaylandIntegration();
 
+    void applyType();
 
     Dialog *q;
     Plasma::Types::Location location;
@@ -241,13 +247,13 @@ void DialogPrivate::updateTheme()
             frameSvgItem->setImagePath(QStringLiteral("dialogs/background"));
         }
 
-        KWindowEffects::enableBlurBehind(q->winId(), theme.blurBehindEnabled(), frameSvgItem->frameSvg()->mask());
+        KWindowEffects::enableBlurBehind(q->winId(), theme.blurBehindEnabled(), frameSvgItem->mask());
 
         KWindowEffects::enableBackgroundContrast(q->winId(), theme.backgroundContrastEnabled(),
                 theme.backgroundContrast(),
                 theme.backgroundIntensity(),
                 theme.backgroundSaturation(),
-                frameSvgItem->frameSvg()->mask());
+                frameSvgItem->mask());
 
         if (KWindowSystem::compositingActive()) {
             if (hasMask) {
@@ -256,7 +262,7 @@ void DialogPrivate::updateTheme()
             }
         } else {
             hasMask = true;
-            q->setMask(frameSvgItem->frameSvg()->mask());
+            q->setMask(frameSvgItem->mask());
         }
         if (q->isVisible()) {
             DialogShadows::self()->addWindow(q, frameSvgItem->enabledBorders());
@@ -308,7 +314,7 @@ void DialogPrivate::updateVisibility(bool visible)
         }
     }
 
-    if (!q->flags().testFlag(Qt::ToolTip) && type != Dialog::Notification) {
+    if (!q->flags().testFlag(Qt::ToolTip) && type != Dialog::Notification && type != Dialog::CriticalNotification) {
         KWindowEffects::SlideFromLocation slideLocation = KWindowEffects::NoEdge;
 
         switch (location) {
@@ -335,17 +341,7 @@ void DialogPrivate::updateVisibility(bool visible)
     if (visible) {
         q->raise();
 
-        if (type != Dialog::Normal) {
-            KWindowSystem::setType(q->winId(), (NET::WindowType)type);
-        } else {
-            q->setFlags(Qt::FramelessWindowHint | q->flags());
-        }
-
-        if (type == Dialog::Dock || type == Dialog::Notification || type == Dialog::OnScreenDisplay) {
-            KWindowSystem::setOnAllDesktops(q->winId(), true);
-        } else {
-            KWindowSystem::setOnAllDesktops(q->winId(), false);
-        }
+        applyType();
     }
 }
 
@@ -709,7 +705,7 @@ void DialogPrivate::setupWaylandIntegration()
     }
 
     using namespace KWayland::Client;
-    PlasmaShell *interface = DialogShadows::self()->waylandPlasmaShellInterface();
+    PlasmaShell *interface = WaylandIntegration::self()->waylandPlasmaShell();
     if (!interface) {
         return;
     }
@@ -721,6 +717,80 @@ void DialogPrivate::setupWaylandIntegration()
 #endif
 }
 
+void DialogPrivate::applyType()
+{
+    if (type != Dialog::Normal) {
+        /*QXcbWindowFunctions::WmWindowType*/ int wmType = 0;
+
+#if HAVE_X11
+        if (KWindowSystem::isPlatformX11()) {
+            switch (type) {
+                case Dialog::Normal:
+                    Q_UNREACHABLE();
+                    break;
+                case Dialog::Dock:
+                    wmType = QXcbWindowFunctions::WmWindowType::Dock;
+                    break;
+                case Dialog::DialogWindow:
+                    wmType = QXcbWindowFunctions::WmWindowType::Dialog;
+                    break;
+                case Dialog::PopupMenu:
+                    wmType = QXcbWindowFunctions::WmWindowType::PopupMenu;
+                    break;
+                case Dialog::Tooltip:
+                    wmType = QXcbWindowFunctions::WmWindowType::Tooltip;
+                    break;
+                case Dialog::Notification:
+                    wmType = QXcbWindowFunctions::WmWindowType::Notification;
+                    break;
+                case Dialog::OnScreenDisplay:
+                case Dialog::CriticalNotification:
+                    // Not supported by Qt
+                    break;
+            }
+
+            if (wmType) {
+                QXcbWindowFunctions::setWmWindowType(q, static_cast<QXcbWindowFunctions::WmWindowType>(wmType));
+            }
+        }
+#endif
+
+        if (!wmType) {
+            KWindowSystem::setType(q->winId(), static_cast<NET::WindowType>(type));
+        }
+    } else {
+        q->setFlags(Qt::FramelessWindowHint | q->flags());
+    }
+
+    //an OSD can't be a Dialog, as qt xcb would attempt to set a transient parent for it
+    //see bug 370433
+    if (type == Dialog::OnScreenDisplay) {
+        q->setFlags((q->flags() & ~Qt::Dialog) | Qt::Window);
+    }
+
+    if (backgroundHints == Dialog::NoBackground) {
+        frameSvgItem->setImagePath(QString());
+    } else {
+        if (type == Dialog::Tooltip) {
+            frameSvgItem->setImagePath(QStringLiteral("widgets/tooltip"));
+        } else {
+            frameSvgItem->setImagePath(QStringLiteral("dialogs/background"));
+        }
+    }
+
+    if (type == Dialog::Dock || type == Dialog::Notification || type == Dialog::OnScreenDisplay || type == Dialog::CriticalNotification) {
+        KWindowSystem::setOnAllDesktops(q->winId(), true);
+    } else {
+        KWindowSystem::setOnAllDesktops(q->winId(), false);
+    }
+
+#if HAVE_KWAYLAND
+    if (shellSurface) {
+        shellSurface->setPanelTakesFocus(!q->flags().testFlag(Qt::WindowDoesNotAcceptFocus));
+    }
+#endif
+}
+
 
 Dialog::Dialog(QQuickItem *parent)
     : QQuickWindow(parent ? parent->window() : nullptr),
@@ -741,8 +811,8 @@ Dialog::Dialog(QQuickItem *parent)
         }
     });
 
-    connect(this, SIGNAL(visibleChanged(bool)),
-            this, SIGNAL(visibleChangedProxy()));
+    connect(this, &QWindow::visibleChanged,
+            this, &Dialog::visibleChangedProxy);
     connect(this, SIGNAL(visibleChanged(bool)),
             this, SLOT(updateInputShape()));
     connect(this, SIGNAL(outputOnlyChanged()),
@@ -898,7 +968,7 @@ QPoint Dialog::popupPosition(QQuickItem *item, const QSize &size)
     }
 
     //if the item is in a dock or in a window that ignores WM we want to position the popups outside of the dock
-    const KWindowInfo winInfo = KWindowSystem::windowInfo(item->window()->winId(), NET::WMWindowType);
+    const KWindowInfo winInfo(item->window()->winId(), NET::WMWindowType);
     const bool outsideParentWindow = ((winInfo.windowType(NET::AllTypesMask) == NET::Dock) || (item->window()->flags() & Qt::X11BypassWindowManagerHint))
             && item->window()->mask().isNull();
 
@@ -1021,6 +1091,7 @@ QObject *Dialog::margins() const
 void Dialog::setFramelessFlags(Qt::WindowFlags flags)
 {
     setFlags(Qt::FramelessWindowHint | flags);
+    d->applyType();
     emit flagsChanged();
 }
 
@@ -1070,33 +1141,7 @@ void Dialog::setType(WindowType type)
     }
 
     d->type = type;
-    if (d->type != Normal) {
-        KWindowSystem::setType(winId(), (NET::WindowType)type);
-    } else {
-        setFlags(Qt::FramelessWindowHint | flags());
-    }
-    //an OSD can't be a Dialog, as qt xcb would attempt to set a transient parent for it
-    //see bug 370433
-    if (type == OnScreenDisplay) {
-        setFlags((flags() & ~Qt::Dialog) | Qt::Window);
-    }
-
-    if (d->backgroundHints == Dialog::NoBackground) {
-        d->frameSvgItem->setImagePath(QString());
-    } else {
-        if (d->type == Tooltip) {
-            d->frameSvgItem->setImagePath(QStringLiteral("widgets/tooltip"));
-        } else {
-            d->frameSvgItem->setImagePath(QStringLiteral("dialogs/background"));
-        }
-    }
-
-    if (type == Dock || type == Notification || type == OnScreenDisplay) {
-        KWindowSystem::setOnAllDesktops(winId(), true);
-    } else {
-        KWindowSystem::setOnAllDesktops(winId(), false);
-    }
-
+    d->applyType();
     emit typeChanged();
 }
 
@@ -1235,11 +1280,24 @@ bool Dialog::event(QEvent *event)
         case QEvent::Wheel: {
             QWheelEvent *we = static_cast<QWheelEvent *>(event);
 
-            if (!d->mainItemContainsPosition(we->pos())) {
-                QWheelEvent we2(d->positionAdjustedForMainItem(we->pos()),
-                                d->positionAdjustedForMainItem(we->pos()) + position(),
-                                we->pixelDelta(), we->angleDelta(), we->delta(),
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+            const QPoint pos = we->pos();
+#else
+            const QPoint pos = we->position().toPoint();
+#endif
+
+            if (!d->mainItemContainsPosition(pos)) {
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+                QWheelEvent we2(d->positionAdjustedForMainItem(pos),
+                                d->positionAdjustedForMainItem(pos) + position(),
+                                we->pixelDelta(), we->angleDelta(), we->angleDelta().y(),
                                 we->orientation(), we->buttons(), we->modifiers(), we->phase());
+#else
+                QWheelEvent we2(d->positionAdjustedForMainItem(pos),
+                                d->positionAdjustedForMainItem(pos) + position(),
+                                we->pixelDelta(), we->angleDelta(),
+                                we->buttons(), we->modifiers(), we->phase(), false /*inverted*/);
+#endif
 
                 if (isVisible()) {
                     QCoreApplication::sendEvent(this, &we2);
