@@ -73,7 +73,7 @@ void ViewPrivate::setContainment(Plasma::Containment *cont)
     Plasma::Types::FormFactor oldForm = formFactor();
 
     if (containment) {
-        QObject::disconnect(containment, 0, q, 0);
+        QObject::disconnect(containment, nullptr, q, nullptr);
         QObject *oldGraphicObject = containment->property("_plasma_graphicObject").value<QObject *>();
         if (oldGraphicObject) {
 //             qDebug() << "Old graphics Object:" << oldGraphicObject << "Old containment" << containment.data();
@@ -119,8 +119,7 @@ void ViewPrivate::setContainment(Plasma::Containment *cont)
 //         qDebug() << "using as graphic containment" << graphicObject << containment.data();
 
         //by resizing before adding, it will avoid some resizes in most cases
-        graphicObject->setProperty("width", q->width());
-        graphicObject->setProperty("height", q->height());
+        graphicObject->setSize(q->size());
         graphicObject->setParentItem(q->rootObject());
         if (q->rootObject()) {
             q->rootObject()->setProperty("containment", QVariant::fromValue(graphicObject));
@@ -159,6 +158,7 @@ void ViewPrivate::showConfigurationInterface(Plasma::Applet *applet)
             configView->hide();
             configView->deleteLater();
         } else {
+            configView->raise();
             configView->requestActivate();
             return;
         }
@@ -197,8 +197,9 @@ View::View(Plasma::Corona *corona, QWindow *parent)
         KDeclarative::KDeclarative kdeclarative;
         kdeclarative.setDeclarativeEngine(engine());
         //binds things like kconfig and icons
-        kdeclarative.setTranslationDomain("plasma_shell_" + pkg.metadata().pluginId());
-        kdeclarative.setupBindings();
+        kdeclarative.setTranslationDomain(QStringLiteral("plasma_shell_") + pkg.metadata().pluginId());
+        kdeclarative.setupContext();
+        KDeclarative::KDeclarative::setupEngine(engine()); // ### how to make sure to do this only once per engine?
     } else {
         qWarning() << "Invalid home screen package";
     }
